/******************************************************************************/
/*                                                                            */
/*   ServoHV.h                                                                */
/*   v 1.8                                                                    */
/*                                                                            */
/*   (c) 2010 Milo Juhs                                                     */
/*                                                                            */
/*   API pre ovladanie horizontalnej a vertikalnej polohy pomocou dvoch       */
/*   servo motorov Hitec HS-422. Jeden motor sluzi na nastavenie horizontalnej*/
/*   a druhy vertikalnej polohy.                                              */
/*                                                                            */
/******************************************************************************/


/*
################################ ZOZNAM FUNKCII ################################
 h_abs_pos()
 | |_____|
 |    |
 |    |___________  (abs_pos)    nastavenie absolutnej polohy v stupnoch
 |                  (left_pos)   nastavenie uplne vlavo
 |                  (top_pos)    nastavenie uplne hore
 |                  (center_pos) nastavenie stredovej polohy
 |                  (right_pos)  nastavenie uplne vpravo
 |                  (bottom_pos) nastavenie uplne dole
 |                  (rel_pos)    relativne nastavenie polohy v stupnoch
 |                  (set_cal)    zapis novych premennych do EEPROM
 |                  (get_cal)    nacitanie parametrov z EEPROM do pamate
 |                  (get_deg)    vrati nastavenu polohu v stupnoch
 |                  (_start_intro_1)    demo test 1
 |                  (_start_intro_2)    demo test 2
 |
 |________________  (h) horizontalne servo
                    (v) vertikalne servo
                    ( ) obe serva
################################################################################
*/

/*
Poznamky:
*1* Knihovna predpoklada budenie LED diod optoclenov katodou.
    Anoda musi byt pripojena na kladne napajacie napatie 5 V
*2* V opacnom pripade je nutne prehodit makra (H/V)_ON, (H/V)_OFF
*3* Kalibracne funkcie automaticky zakazu globalne prerusenie
*4* Konstanty su nastavene bez zataze, po zatazeni sa mozu zmenit
*/


#ifndef F_CPU
#define F_CPU   16000000UL
#endif

#ifndef DDR
#define DDR(n)  (*(&n -1))
#endif

#define H_PORT  PORTD
#define V_PORT  PORTD
#define H_PIN   7
#define V_PIN   6

#define H_ON    H_PORT |= (1 << H_PIN)
#define H_OFF   H_PORT &= ~(1 << H_PIN)
#define V_ON    V_PORT |= (1 << V_PIN)
#define V_OFF   V_PORT &= ~(1 << V_PIN)
// pocet cyklov k dokonceniu periody
#define PERIOD_CYCLES   20  
// konverzia unsigned int na unsigned char
#define G_uHIGH(x)      (unsigned char)(x>>8)
#define G_uLOW(x)       (unsigned char)(x)

// *** Adresy defaultnych hodnot premennych v EEPROM ***
#define ADDR_L_uiTopOcrV        0x00
#define ADDR_H_uiTopOcrV        0x01
#define ADDR_L_uiBottomOcrV     0x02
#define ADDR_H_uiBottomOcrV     0x03
#define ADDR_ucTopDegV          0x04
#define ADDR_ucBottomDegV       0x05
#define ADDR_L_uiLastPosV       0x06
#define ADDR_H_uiLastPosV       0x07

#define ADDR_L_uiLeftOcrH       0x08
#define ADDR_H_uiLeftOcrH       0x09
#define ADDR_L_uiRightOcrH      0x0A
#define ADDR_H_uiRightOcrH      0x0B
#define ADDR_ucLeftDegH         0x0C
#define ADDR_ucRightDegH        0x0D
#define ADDR_L_uiLastPosH       0x0E
#define ADDR_H_uiLastPosH       0x0F

#define ADDR_L_uiBaseOcr        0x10
#define ADDR_H_uiBaseOcr        0x11
#define ADDR_ucSetCycles        0x12


// ### INCLUDES ###

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/eeprom.h>

// ### GLOBALS ###

        // kalibracne konstanty platne pre F_OSC = 8 MHz!!!
/*
// *** Vertikalny pohyb ***
static unsigned int uiTopOcrV = 600;        // 560 us pre hornu polohu
static unsigned int uiBottomOcrV = 1500;    // 1430 us pre spodnu polohu
static unsigned char ucTopDegV = 90;        // max. horna poloha v stupnoch
static unsigned char ucBottomDegV = 0;      // max. dolna poloha v stupnoch
static unsigned int uiConstV;               // konst. pre prepocet na stupne
static unsigned int uiLastPosV = 1000;      // nastavena poloha
// *** Horizontalny pohyb ***
static unsigned int uiLeftOcrH = 600;       // 560 us pre lavu polohu
static unsigned int uiRightOcrH = 2350;     // 2300 us pre pravu polohu
static unsigned char ucLeftDegH = 180;      // max. vpravo v stupnoch
static unsigned char ucRightDegH = 0;       // max. vlavo v stupnoch
static unsigned int uiConstH;               // konst. pre prepocet na stupne
static unsigned int uiLastPosH=1000;        // nastavena poloha
// *** Oba motorceky ***
static unsigned char ucToggleHV=0;          // prepinanie H a V serva
// *** Timer ***
static unsigned int uiBaseOcr=1023;     // zakladna casova jednotka
static unsigned char ucSetCycles=40;    // pocet period do ustalenia polohy
volatile unsigned char ucCount = 0;     // pocita do PERIOD_CYCLES
volatile unsigned char ucSetTime = 0;   // pocita do ucSetCycles
*/

        // kalibracne konstanty platne pre F_OSC = 16 MHz!!! - netestovane

// *** Vertikalny pohyb ***
static unsigned int uiTopOcrV = 1200;       // 560 us pre hornu polohu
static unsigned int uiBottomOcrV = 3000;    // 1430 us pre spodnu polohu
static unsigned char ucTopDegV = 90;        // max. horna poloha v stupnoch
static unsigned char ucBottomDegV = 0;      // max. dolna poloha v stupnoch
static unsigned int uiConstV;               // konst. pre prepocet na stupne
static unsigned int uiLastPosV = 2000;      // nastavena poloha
// *** Horizontalny pohyb ***
static unsigned int uiLeftOcrH = 1200;      // 560 us pre lavu polohu
static unsigned int uiRightOcrH = 4700;     // 2300 us pre pravu polohu
static unsigned char ucLeftDegH = 180;      // max. vpravo v stupnoch
static unsigned char ucRightDegH = 0;       // max. vlavo v stupnoch
unsigned int uiConstH;              // konst. pre prepocet na stupne
static unsigned int uiLastPosH=2000;        // nastavena poloha
// *** Oba motorceky ***
static unsigned char ucToggleHV=0;          // prepinanie H a V serva
// *** Timer ***
static unsigned int uiBaseOcr=2048;     // zakladna casova jednotka
static unsigned char ucSetCycles=40;    // pocet period do ustalenia polohy
volatile unsigned char ucCount = 0;     // pocita do PERIOD_CYCLES
volatile unsigned char ucSetTime = 0;   // pocita do ucSetCycles


// ### PROTOTYPES ###

// *** Spolocne funkcie ***
    void _servo_init(void);         //  ok
    unsigned char _abs_pos(unsigned char setPosH, unsigned char setPosV);
    unsigned char _rel_pos(char setOffsetH, char setOffsetV);
    unsigned char _center_pos(void);
    void _set_cal(char write);
    void _get_cal(char read);
    void _start_intro_1(void);
    void _start_intro_2(void);
    unsigned int h_get_deg(void);
    unsigned int v_get_deg(void);
// *** Vertikalny pohyb ***
    void v_set_const(void);
    unsigned char v_abs_pos(unsigned char setPosV);
    unsigned char v_top_pos(void);
    unsigned char v_center_pos(void);
    unsigned char v_bottom_pos(void);
    unsigned char v_rel_pos(char setOffsetV);
// *** Horizontalny pohyb ***
    void h_set_const(void);
    unsigned char h_abs_pos(unsigned char setPosH);
    unsigned char h_left_pos(void);
    unsigned char h_center_pos(void);
    unsigned char h_right_pos(void);
    unsigned char h_rel_pos(char setOffsetH);
// *** Kalibracne funkcie - praca s premennymi ***
    void _set_uiBaseOcr(unsigned int val);
    void _set_ucSetCycles(unsigned char val);
    void _set_uiTopOcrV(unsigned int val);
    void _set_uiBottomOcrV(unsigned int val);
    void _set_uiLastPosV(unsigned int val);
    void _set_uiLeftOcrH(unsigned int val);
    void _set_uiRightOcrH(unsigned int val);
    void _set_uiLastPosH(unsigned int val);
    void _set_ucTopDegV(unsigned char val);
    void _set_ucBottomDegV(unsigned char val);
    void _set_ucLeftDegH(unsigned char val);
    void _set_ucRightDegH(unsigned char val);
    unsigned int _get_uiBaseOcr(void);
    unsigned char _get_ucSetCycles(void);
    unsigned int _get_uiTopOcrV(void);
    unsigned int _get_uiBottomOcrV(void);
    unsigned int _get_uiLastPosV(void);
    unsigned int _get_uiLeftOcrH(void);
    unsigned int _get_uiRightOcrH(void);
    unsigned int _get_uiLastPosH(void);
    unsigned char _get_ucTopDegV(void);
    unsigned char _get_ucBottomDegV(void);
    unsigned char _get_ucLeftDegH(void);
    unsigned char _get_ucRightDegH(void);
        
// ### FUNCTIONS ###

// *** Spolocne funkcie ***

/*
# Inicializacia nastaveni
#   Pociatocne nastavenie portov, registrov a premennych
*/

void _servo_init(void)
{
// Ak sa pouziva kalibracia je mozne nacitat nastavenia automaticky v ramci
// inicializacie. V takom pripade treba zrusit nasledujuci komentar:
//  _get_cal(1);

    DDR(H_PORT)|=(1<<H_PIN);
    DDR(V_PORT)|=(1<<V_PIN);

    uiConstV = ((uiBottomOcrV-uiTopOcrV)*10) / (ucTopDegV-ucBottomDegV);
    uiConstH = ((uiRightOcrH-uiLeftOcrH)*10) / (ucLeftDegH-ucRightDegH);

    TCCR1B |= (1<<WGM12) | (1<<CS11);
    OCR1A = 1025;
    sei();
}


/*
# Kalibracia parametrov - ulozenie do EEPROM
#   Funkcia ulozi globalne premenne do EEPROM. 
#       >> write != 0:  zapise aktualne hodnoty premennych do EEPROM.
#       >> write = 0:   vratane aktualnej polohy
#   Vztahuje sa na: uiTopOcrV, uiBottomOcrV, ucTopDegV, ucBottomDegV
#                   uiLeftOcrH, uiRightOcrH, ucLeftDegH, ucRightDegH,
#                   uiBaseOcr, ucSetCycles a pre write = 2
#
#   UPOZORNENIE: funkcia vypina globalne prerusenie!!!
#
*/

void _set_cal(char write)
{
    cli();
// Vertikalne premenne
    _EEPUT(ADDR_L_uiTopOcrV,G_uLOW(uiTopOcrV));         // uiTopOcrV
    _EEPUT(ADDR_H_uiTopOcrV,G_uHIGH(uiTopOcrV));

    _EEPUT(ADDR_L_uiBottomOcrV,G_uLOW(uiBottomOcrV));   // uiBottomOcrV
    _EEPUT(ADDR_H_uiBottomOcrV,G_uHIGH(uiBottomOcrV));

    _EEPUT(ADDR_ucTopDegV,ucTopDegV);                   // ucTopDegV
    _EEPUT(ADDR_ucBottomDegV,ucBottomDegV);             // ucBottomDegV

    if (!write)                                         // uiLastPosV
    {
        _EEPUT(ADDR_L_uiLastPosV,G_uLOW(uiLastPosV));
        _EEPUT(ADDR_H_uiLastPosV,G_uHIGH(uiLastPosV));
    }

// Horizontalne premenne
    _EEPUT(ADDR_L_uiLeftOcrH,G_uLOW(uiLeftOcrH));       // uiLeftOcrH
    _EEPUT(ADDR_H_uiLeftOcrH,G_uHIGH(uiLeftOcrH));

    _EEPUT(ADDR_L_uiRightOcrH,G_uLOW(uiRightOcrH));     // uiRightOcrH
    _EEPUT(ADDR_H_uiRightOcrH,G_uHIGH(uiRightOcrH));

    _EEPUT(ADDR_ucLeftDegH,ucLeftDegH);                 // ucLeftDegH
    _EEPUT(ADDR_ucRightDegH,ucRightDegH);               // ucRightDegH

    if (!write)
    {
        _EEPUT(ADDR_L_uiLastPosH,G_uLOW(uiLastPosH));   // uiLastPosH
        _EEPUT(ADDR_H_uiLastPosH,G_uHIGH(uiLastPosH));
    }

    _EEPUT(ADDR_L_uiBaseOcr,G_uLOW(uiBaseOcr));         // uiBaseOcr
    _EEPUT(ADDR_H_uiBaseOcr,G_uHIGH(uiBaseOcr));

    _EEPUT(ADDR_ucSetCycles,ucSetCycles);               // ucSetCycles

}


/*
# Kalibracia parametrov - nacitanie z EEPROM
#   Po resete je mozne bud nacitat premenne z EEPROM funkciou _servo_init(),
#   alebo vratit defalutne parametre do EEPROM (_set_cal(0) pred _servo_init()).
#       >> read != 0:   nacita parametre z EEPROM do pamate
#       >> read = 0:    vratane defaultnej polohy
#   Vztahuje sa na: uiTopOcrV, uiBottomOcrV, ucTopDegV, ucBottomDegV
#                   uiLeftOcrH, uiRightOcrH, ucLeftDegH, ucRightDegH,
#                   uiBaseOcr, ucSetCycles a pre write = 2
#
#   UPOZORNENIE: funkcia vypina globalne prerusenie!!!
#
*/

void _get_cal(char read)
{
    unsigned int temp;
//  unsigned char *tmp;     // mensi kod ale s warningom
    cli();
    // Vertikalne premenne
    _EEGET(uiTopOcrV,ADDR_L_uiTopOcrV);         // uiTopOcrV
    _EEGET(temp,ADDR_H_uiTopOcrV);
    uiTopOcrV |= (temp<<8);

//  tmp = &(uiTopOcrV)-1;
//  _EEGET(uiTopOcrV,ADDR_L_uiTopOcrV);         // uiTopOcrV
//  _EEGET(*tmp,ADDR_H_uiTopOcrV);



    _EEGET(uiBottomOcrV,ADDR_L_uiBottomOcrV);   // uiBottomOcrV
    _EEGET(temp,ADDR_H_uiBottomOcrV);
    uiBottomOcrV |= (temp<<8);

    _EEGET(ucTopDegV,ADDR_ucTopDegV);           // ucTopDegV

    _EEGET(ucBottomDegV,ADDR_ucBottomDegV);     // ucBottomDegV

    if (!read)                                  // uiLastPosV
    {
        _EEGET(uiLastPosV,ADDR_L_uiLastPosV);
        _EEGET(temp,ADDR_H_uiLastPosV);
        uiLastPosV |= (temp<<8);
    }

// Horizontalne premenne
    _EEGET(uiLeftOcrH,ADDR_L_uiLeftOcrH);           // uiLeftOcrH
    _EEGET(uiLeftOcrH,ADDR_H_uiLeftOcrH);
    uiBaseOcr |= (temp<<8);

    _EEGET(uiRightOcrH,ADDR_L_uiRightOcrH);         // uiRightOcrH
    _EEGET(uiRightOcrH,ADDR_H_uiRightOcrH);
    uiRightOcrH |= (temp<<8);

    _EEGET(ucLeftDegH,ADDR_ucLeftDegH);             // ucLeftDegH

    _EEGET(ucRightDegH,ADDR_ucRightDegH);           // ucRightDegH

    if (!read)                                      // uiLastPosH
    {
        _EEGET(uiLastPosH,ADDR_L_uiLastPosH);
        _EEGET(uiLastPosH,ADDR_H_uiLastPosH);
        uiLastPosH |= (temp<<8);
    }

    _EEGET(uiBaseOcr,ADDR_L_uiBaseOcr);             // uiBaseOcr
    _EEGET(temp,ADDR_H_uiBaseOcr);
    uiBaseOcr |= (temp<<8);

    _EEGET(ucSetCycles,ADDR_ucSetCycles);           // ucSetCycles
}


/*
# Funkcie na pracu s globalnymi premennymi
#
*/

    void _set_uiBaseOcr(unsigned int val)
    {
        uiBaseOcr = val;        
    }
    void _set_ucSetCycles(unsigned char val)
    {
        ucSetCycles = val;        
    }    
    void _set_uiTopOcrV(unsigned int val)
    {
        uiTopOcrV = val;        
    }    
    void _set_uiBottomOcrV(unsigned int val)
    {
        uiBottomOcrV = val;        
    }    
    void _set_uiLastPosV(unsigned int val)
    {
        uiLastPosV = val;        
    }    
    void _set_uiLeftOcrH(unsigned int val)
    {
        uiLeftOcrH = val;        
    }    
    void _set_uiRightOcrH(unsigned int val)
    {
        uiRightOcrH = val;        
    }    
    void _set_uiLastPosH(unsigned int val)
    {
        uiLastPosH = val;        
    }    
    void _set_ucTopDegV(unsigned char val)
    {
        ucTopDegV = val;        
    }    
    void _set_ucBottomDegV(unsigned char val)
    {
        ucBottomDegV = val;        
    }    
    void _set_ucLeftDegH(unsigned char val)
    {
        ucLeftDegH = val;        
    }    
    void _set_ucRightDegH(unsigned char val)
    {
        ucRightDegH = val;        
    }
    
    unsigned int _get_uiBaseOcr(void)
    {
        return uiBaseOcr;
    }
    unsigned char _get_ucSetCycles(void)
    {
        return ucSetCycles;
    }
    unsigned int _get_uiTopOcrV(void)
    {
        return uiTopOcrV;
    }
    unsigned int _get_uiBottomOcrV(void)
    {
        return uiBottomOcrV;
    }
    unsigned int _get_uiLastPosV(void)
    {
        return uiLastPosV;
    }
    unsigned int _get_uiLeftOcrH(void)
    {
        return uiLeftOcrH;
    }
    unsigned int _get_uiRightOcrH(void)
    {
        return uiRightOcrH;
    }
    unsigned int _get_uiLastPosH(void)
    {
        return uiLastPosH;
    }
    unsigned char _get_ucTopDegV(void)
    {
        return ucTopDegV;
    }
    unsigned char _get_ucBottomDegV(void)
    {
        return ucBottomDegV;
    }
    unsigned char _get_ucLeftDegH(void)
    {
        return ucLeftDegH;
    }
    unsigned char _get_ucRightDegH(void)
    {
        return ucRightDegH;
    }
/*
# Nastavenie absolutnej polohy v oboch smeroch
#   >> setPosH: celociselny horizontalny uhol zadavany v stupnoch, rozsah
#               je obmadzeny premennymi ucRightH a ucLeftDegH
#   >> setPosV: celociselny vertikalny uhol zadavany v stupnoch, rozsah
#               je obmadzeny premennymi ucBottomV a ucTopDegV
#   << 0:   poloha nastavena
#   << 1:   hodnota mimo rozsahu
*/

unsigned char _abs_pos(unsigned char setPosH, unsigned char setPosV)
{
    if ((setPosH <= ucLeftDegH) && (setPosH >= ucRightDegH))
    {
    // WAIT..
        while (bit_is_set(TIMSK,OCIE1A));
        uiLastPosH = uiLeftOcrH+(uiConstH*setPosH)/10;
        ucToggleHV = 1;
        TIMSK |= (1<<OCIE1A);
    }


    else
        return 1;

    if ((setPosV <= ucTopDegV) && (setPosV >= ucBottomDegV))
    {
    // WAIT..
        while (bit_is_set(TIMSK,OCIE1A));
        uiLastPosV = uiTopOcrV+(uiConstV * (ucTopDegV-setPosV))/10;
        ucToggleHV = 0;
        TIMSK |= (1<<OCIE1A);
    }
    else
        return 1;

    return 0;
}


/*
# Nastavenie relativnej polohy v oboch smeroch
#   >> setOffsetH:  cele cislo v stupnoch, o ktore sa zmeni horizontalna
#                   poloha z rozsahu <-128,127>, pricom sucet s aktualnou
#                   polohou serva musi byt z intervalu <ucRightDegH,ucLeftDegH>
#   >> setOffsetV:  cele cislo v stupnoch, o ktore sa zmeni vertikalna poloha
#                   z rozsahu <-128,127>, pricom sucet s aktualnou polohou serva
#                   musi byt z intervalu <ucBottomDegV,ucTopDegV>
#   << 0:   poloha nastavena
#   << 1:   hodnota mimo rozsahu
*/

unsigned char _rel_pos(char setOffsetH, char setOffsetV)
{
    if (_abs_pos(uiLastPosH + setOffsetH, uiLastPosV + setOffsetV))
        return 1;
    return 0;
}


/*
# Intro 1
#   Otestuje jednotlive polohy serva, najskor horizontalne, potom vertikalne
#   v tomto poradi: Stredna, Prava, Lava, Stredna,
#                   Stredna, Horna, Spodna, Stredna
*/

void _start_intro_1(void)
{
    h_center_pos();
    h_right_pos();
    h_left_pos();
    h_center_pos();

    v_center_pos();
    v_top_pos();
    v_bottom_pos();
    v_center_pos();
}

/*
# Intro 2
#   Otestuje nastavenie polohy serva v tomto poradi:
#                   [90, 0]
#                   {180, 0, 135, 45, 90} horizontalne
#                   {90,45, 0, 90, 0} vertikalne
*/

void _start_intro_2(void)
{
    _abs_pos(90,0);

    h_abs_pos(180);
    h_abs_pos(0);
    h_abs_pos(135);
    h_abs_pos(45);
    h_abs_pos(90);

    v_abs_pos(90);
    v_abs_pos(45);
    v_abs_pos(0);
    v_abs_pos(90);
    v_abs_pos(0);
}


/*
# Nastavenie strednej polohy v oboch osach
#   Vypocita a nastavi strednu horizontalnu aj vertikalnu polohu
#   ako aritmeticky priemer premennych ucLeftDegH, ucRightDegH
#   a ucTopDegV, ucBottomDegV
#   << 0:   poloha nastavena
#   << 1:   hodnota mimo rozsahu
*/

unsigned char _center_pos(void)
{
    if (

    // # stredova poloha pre vertikalny smer je aritmetickym priemerom
    // #  krajnych hodnot:

    // _abs_pos((ucLeftDegH-ucRightDegH) >> 1,(ucTopDegV-ucBottomDegV) >> 1)

    // # stredova poloha pre vertikalny smer je 0:

     _abs_pos((ucLeftDegH-ucRightDegH) >> 1,0)

    // # stredova poloha pre vertikalny smer je 90:

    // _abs_pos((ucLeftDegH-ucRightDegH) >> 1,90)
    ) return 1;
    return 0;
    
}


// *** Vertikalny pohyb ***

/*
# Vypocet konstanty pre vertikalny pohyb
#   Funkciu je mozne pouzit pri zmene krajnej vertikalnej polohy namiesto
#   _servo_init(), v pripade, ze je potrebne prepocitat vertikalnu konstantu
*/

void v_set_const(void)
{
    uiConstV = ((uiBottomOcrV-uiTopOcrV)*10) / (ucTopDegV-ucBottomDegV);    
}


/*
# Nastavenie vertikalnej polohy
#   >> setPosV: celociselny horizontalny uhol zadavany v stupnoch, rozsah
#               je obmadzeny premennymi ucBottomDegH a ucTopDegH
#   << 0:   poloha nastavena
#   << 1:   hodnota mimo rozsahu
*/

unsigned char v_abs_pos(unsigned char setPosV)
{
    if ((setPosV <= ucTopDegV) && (setPosV >= ucBottomDegV))
    {
    // WAIT..
        while (bit_is_set(TIMSK,OCIE1A));
        uiLastPosV = uiTopOcrV+(uiConstV * (ucTopDegV-setPosV))/10;
        ucToggleHV = 0;
        TIMSK |= (1<<OCIE1A);
    }

    else 
        return 1;
    return 0;
}

/*
# Nastavenie relativnej vertikalnej polohy
#   >> setOffsetV:  cele cislo v stupnoch, o ktore sa zmeni vertikalna poloha
#                   z rozsahu <-128,127>, pricom sucet s aktualnou polohou serva
#                   musi byt z intervalu <ucBottomDegV,ucTopDegV>
#   << 0:   poloha nastavena
#   << 1:   hodnota mimo rozsahu
*/

unsigned char v_rel_pos(char setOffsetV)
{
    if (v_abs_pos(uiLastPosV + setOffsetV))
        return 1;
    return 0;
}

/*
# Nastavenie krajnej vertikalnej polohy - hore
#   Polohu nastavi ako ucTopDegV
#   << 0:   poloha nastavena
#   << 1:   hodnota mimo rozsahu
*/

unsigned char v_top_pos(void)
{
    if (v_abs_pos(ucTopDegV))
        return 1;
    return 0;
}


/*
# Nastavenie stredovej vertikalnej polohy
#   Polohu vypocita ako aritmeticky priemer premennych ucTopDegV,
#   ucBottomDegV
#   << 0:   poloha nastavena
#   << 1:   hodnota mimo rozsahu
*/

unsigned char v_center_pos(void)
{
    if (
    // # stredova poloha pre vertikalny smer je aritmetickym priemerom
    // # krajnych hodnot:

    // v_abs_pos((ucTopDegV-ucBottomDegV) >> 1)


    // # stredova poloha pre vertikalny smer je 0:

     v_abs_pos(0)

    // # stredova poloha pre vertikalny smer je 90:

    // v_abs_pos(90)
    ) return 1;
    return 0;   
}


/*
# Nastavenie krajnej vertikalnej polohy - dole
#   Polohu nastavi ako ucBottomDegV
#   << 0:   poloha nastavena
#   << 1:   hodnota mimo rozsahu
*/

unsigned char v_bottom_pos(void)
{
    if (v_abs_pos(ucBottomDegV))
        return 1;
    return 0;
}


/*
# Zistenie nastavenej vertikalnej polohy v stupnoch
#   Prepocita poslednu nastavenu polohu na stupne, vrati hodnotu 10x
#   vacsiu takze je mozne ziskat aj jedno desatinne miesto, aj ked
#   polohu motorcekov tak presne nastavit nejde
#   << poloha v stupnoch (napr. 1253: 1253/10,1253%10 = 125,3)
*/

unsigned int v_get_deg(void)
{
    return (ucTopDegV-(10*(uiLastPosV-uiTopOcrV)/uiConstV));
}


// *** Horizontalny pohyb ***
/*
# Vypocet konstanty pre horizontalny pohyb
#   Funkciu je mozne pouzit pri zmene krajnej horizontalnej polohy namiesto
#   _servo_init(), v pripade, ze je potrebne prepocitat horizontalnu konstantu
*/

void h_set_const(void)
{
    uiConstH = ((uiRightOcrH - uiLeftOcrH) * 10) / (ucLeftDegH - ucRightDegH);
}


/*
# Nastavenie horizontalnej polohy
#   >> setPosH: celociselny horizontalny uhol zadavany v stupnoch, rozsah
#               je obmadzeny premennymi ucRightDegH a ucLeftDegH
#   << 0:   poloha nastavena
#   << 1:   hodnota mimo rozsahu
*/

unsigned char h_abs_pos(unsigned char setPosH)
{
    if ((setPosH <= ucLeftDegH) && (setPosH >= ucRightDegH))
    {
        while (bit_is_set(TIMSK,OCIE1A));               // WAIT..
        uiLastPosH = uiLeftOcrH + (uiConstH*setPosH)/10;
        ucToggleHV = 1;
        TIMSK |= (1<<OCIE1A);
    }
    else
        return 1;
    return 0;
}


/*
# Nastavenie relativnej horizontalnej polohy
#   >> setOffsetH:  cele cislo v stupnoch, o ktore sa zmeni horizontalna
#                   poloha z rozsahu <-128,127>, pricom sucet s aktualnou
#                   polohou serva musi byt z intervalu <ucRightDegH,ucLeftDegH>
#   << 0:   poloha nastavena
#   << 1:   hodnota mimo rozsahu
*/

unsigned char h_rel_pos(char setOffsetH)
{
    if (h_abs_pos(uiLastPosH + setOffsetH))
        return 1;
    return 0;
}


/*
# Nastavenie krajnej horizontalnej polohy - vlavo
#   Polohu nastavi ako ucLeftDegH
#   << 0:   poloha nastavena
#   << 1:   hodnota mimo rozsahu
*/

unsigned char h_left_pos(void)
{
    if (h_abs_pos(ucLeftDegH))
        return 1;
    return 0;
}


/*
# Nastavenie stredovej horizontalnej polohy
#   Polohu vypocita ako aritmeticky priemer premennych ucLeftDegH,
#   ucRightDegH
#   << 0:   poloha nastavena
#   << 1:   hodnota mimo rozsahu
*/

unsigned char h_center_pos(void)
{
    if (h_abs_pos((ucLeftDegH-ucRightDegH) >> 1))
        return 1;
    return 0;
}


/*
# Nastavenie krajnej horizontalnej polohy - vpravo
#   Polohu nastavi ako ucRightDegH
#   << 0:   poloha nastavena
#   << 1:   hodnota mimo rozsahu
*/

unsigned char h_right_pos(void)
{
    if (h_abs_pos(ucRightDegH))
        return 1;
    return 0;
}


/*
# Zistenie nastavenej horizontalnej polohy v stupnoch
#   Prepocita poslednu nastavenu polohu na stupne, vrati hodnotu 10x
#   vacsiu takze je mozne ziskat aj jedno desatinne miesto, aj ked
#   polohu motorcekov tak presne nastavit nejde
#   << poloha v stupnoch (napr. 1253: 1253/10,1253%10 = 125,3)
*/

unsigned int h_get_deg(void)
{
    return ((10*(uiLastPosH-uiLeftOcrH))/uiConstH);
}


// ### OBSLUHA PRERUSENIA ###

// TIMER1A in CTC mode

ISR( TIMER1_COMPA_vect )
{
    switch (ucCount++)
    {
    case 0: (ucToggleHV)?(H_OFF):(V_OFF);   // zaciatok periody
            OCR1A = uiBaseOcr;
        break;
    case 1: (ucToggleHV)?(H_ON):(V_ON);     // zapnut
            OCR1A = (ucToggleHV)?(uiLastPosH):(uiLastPosV);
        break;
    case 2: (ucToggleHV)?(H_OFF):(V_OFF);   // vypnut
            OCR1A = uiBaseOcr;
        break;
    case (PERIOD_CYCLES-1):                 // koniec periody
            ucCount = 0;
            OCR1A = uiBaseOcr;
            if (++ucSetTime == ucSetCycles)
                {
                    ucSetTime = 0;
                    TIMSK &= ~(1<<OCIE1A);
                }
        break;
    }
}
